# ***************************************************************************
# Copyright 2019 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************

from cli.commands.cli_command_provider import CliCommandProvider
from .log_level_config_utils import update_config_file
from .log_level_config_utils import read_config_file
import errno
import json
from mako import compat
import nsx_logging
import os

SERVICE_NAME = "nsx-context-mux"
INFO = "info"

lg = nsx_logging.getLogger("cli.commands.common.context_mux_log_level")


# ----------------------------------------------------------------------
# Set nsx-context-mux logging level
# ----------------------------------------------------------------------
class SetContextMuxLoggingLevel(CliCommandProvider):
    def __call__(self, cmd_list):
        level = self.get_arg_value("context-mux-logging-level-arg", cmd_list)
        if self.set_logging_level(SERVICE_NAME, level) == True:
            return "OK\n\n%s" % json.dumps("")
        else:
            return "MSG_SET_CONTEXT_MUX_LOG_LEVEL_FAILED"

    def set_logging_level(self, daemon_name, level):
        #  Updates daemon's config file with specified log level.
        #  Returns true if successful, otherswise returns false.
        if not update_config_file(daemon_name, level):
            return False
        return True


# ----------------------------------------------------------------------
# Get nsx-context-mux logging level
# ----------------------------------------------------------------------
class GetContextMuxLoggingLevel(CliCommandProvider):
    def __call__(self, cmd_list):
        logLevel = self.get_logging_level(SERVICE_NAME)
        if logLevel == False:
            return "MSG_GET_CONTEXT_MUX_LOG_LEVEL_FAILED"
        else:
            return "OK\n\n%s" % json.dumps(logLevel)

    def get_logging_level(self, daemon_name):
        logLevel = read_config_file(daemon_name)
        if logLevel == False:
            return INFO
        else:
            return logLevel
