# ***************************************************************************
# Copyright 2017 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import json
import string

from cli.commands.cli_command_provider import CliCommandProvider
from cli.commands.lcp_common.nsx_proxy.client import get_nsxproxy_cli_output
from cli.utils.ipaddr_utils import ip_int_to_string
from cli.utils.nestdb_utils import invoke_nestdb_command
from cli.utils.node_utils import is_windows
from cli.utils.command_utils import invoke_command


CCP_SESSION_MSG = "vmware.nsx.nestdb.CcpSessionMsg"


class NestDBCcpSessionConvertor():
    @staticmethod
    def get_ccp_session_msg():
        cmd = ["get %s" % CCP_SESSION_MSG]
        return invoke_nestdb_command(cmd)

    @staticmethod
    def parse_ccp_session_msg():
        rc, rv, err = NestDBCcpSessionConvertor.get_ccp_session_msg()
        if rc != 0:
            return (rc, rv, err)

        if rv is None:
            return (0, None, "not exist")

        resp = json.loads(rv)
        if "result" in resp:
            out = [v["value"] for v in resp["result"]]
            return (0, out, None)
        else:
            return (0, None, "not exist")

    @staticmethod
    def ccp_session_msg_convertor(controller):
        entry = {}
        entry["controller_ip"] = ip_int_to_string(controller["ip"]["ipv4"])
        entry["port"] = controller["server_port"]
        entry["ssl"] = "enabled"

        if controller["state"] == "CONNECTED":
            entry["status"] = "connected"
            entry["session_state"] = "up"
        elif controller["state"] == "INVALID":
            entry["status"] = "invalid"
            entry["session_state"] = "null"

        if controller["master"]:
            entry["is_physical_master"] = "true"
            if controller["state"] == "DISCONNECTED":
                entry["status"] = "disconnected"
                entry["session_state"] = "down"
        else:
            entry["is_physical_master"] = "false"
            if controller["state"] == "DISCONNECTED":
                entry["status"] = "not used"
                entry["session_state"] = "null"

        if controller["fqdn"] == "":
            entry["controller_fqdn"] = "NA"
        else:
            entry["controller_fqdn"] = controller["fqdn"]

        return entry

    @staticmethod
    def get_ccp_session_info():
        rc, rv, err = NestDBCcpSessionConvertor.parse_ccp_session_msg()
        if rc != 0:
            return None

        entry_list = []
        if rv is None:
            return json.dumps(entry_list)

        for val in rv:
            if val:
                entry_list.append(
                    NestDBCcpSessionConvertor.ccp_session_msg_convertor(val))

        return json.dumps(entry_list)


class ReloadControllerInfoList(CliCommandProvider):
    def __call__(self, cmd_list):
        config_file = cmd_list[4]
        response = get_nsxproxy_cli_output("controller-info/reload",
                                           [config_file])
        if response is None:
            return "MSG_HOST_CONTROLLER_INFO_RELOAD_FAILED"

        return "OK\n\n%s" % json.dumps(response)


class GetControllerConnectionInfo(CliCommandProvider):
    def __call__(self, cmd_list):
        if is_windows():
            cmd = ["sc.exe", "query", "nsx-proxy"]
        else:
            cmd = ["/etc/init.d/nsx-proxy", "status"]
        rc, out, err = invoke_command(cmd);

        if rc != 0:
            return "MSG_HOST_CONTROLLER_INFO_GET_FAILED"

        if is_windows():
            if out.find("RUNNING") == -1:
                return "MSG_HOST_CONTROLLER_INFO_GET_FAILED"

        ccp_session = NestDBCcpSessionConvertor.get_ccp_session_info()
        if ccp_session is None:
            return "MSG_HOST_CONTROLLER_INFO_GET_FAILED"
        return "OK\n\n%s" % ccp_session
