# Copyright (C) 2020-2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

def report_edge_fw_conn(edge_fw_stats, edge_fw_conn, profile):
    if edge_fw_conn is None and edge_fw_stats is None:
        logger.info("NO EDGE_FW_CONN stats for exporting.")
        return

    precondition = data_store['metrics']['nsx.sha.precondition']

    if edge_fw_stats is not None and edge_fw_stats['value'] is not None:
        logger.debug("EDGE_FW_STATS stats is {}".format(edge_fw_stats['value']))
        logger.debug("EDGE_FW_CONN stats is {}".format(edge_fw_conn['value']))
        # Use specified source and timestamp to export metrics to wavefront
        wavefront_source = edge_fw_stats['value'][0]
        timestamp = edge_fw_stats['timestamp']

        # Get firewall ingress conn info from stats
        fw_stats_sum = {}
        for lr, fw_stats in edge_fw_stats['value'][1].items():
            if fw_stats is None:
                continue

            stats_conn_data = {}
            fw_stats_name = "edge." + lr + ".firewall.connections"

            try:
                conn_data = fw_stats['TCP Half Opened Active/Max'].split('/')
                if len(conn_data) == 2:
                    stats_conn_data['tcp'] = {'halfOpenIngress': int(conn_data[0]),
                                              'max': int(conn_data[1])}
                else:
                    stats_conn_data['tcp'] = {'halfOpenIngress': 0,
                                              'max': 0}
                conn_data = fw_stats['Other Active/Max'].split('/')
                if len(conn_data) == 2:
                    stats_conn_data['others'] = {'ingress': int(conn_data[0]),
                                                 'max': int(conn_data[1])}
                else:
                    stats_conn_data['others'] = {'ingress': 0,
                                                 'max': 0}
                for item in ['ICMP', 'UDP']:
                    conn_data = fw_stats[item + ' Active/Max'].split('/')
                    if len(conn_data) == 2:
                        stats_conn_data[item.lower()] = {'ingress': int(conn_data[0]),
                                                         'max': int(conn_data[1])}
                    else:
                        stats_conn_data[item.lower()] = {'ingress': 0,
                                                         'max': 0}
                fw_stats_sum[fw_stats_name] = stats_conn_data
                if precondition and precondition['value']['tsdb']['reached']:
                    lr_type, lr_name = lr.split('.')
                    entity_type = options_pb2.MetricsEntityType.Value('TIER1') if lr_type == 'T1' else \
                                  options_pb2.MetricsEntityType.Value('TIER0')
                    msg = metrics_agent_rpc_pb2.MetricsAgentMessage(
                          timestamp = int(timestamp),
                          entity = entity_type,
                          entity_id = str(fw_stats['mp_router_id']))
                    tcp = stats_conn_data['tcp']
                    tcp_half_open_ingress_conn = tcp['halfOpenIngress']
                    tcp_max_conn = tcp['max']
                    udp = stats_conn_data['udp']
                    udp_ingress_conn = udp['ingress']
                    udp_max_conn = udp['max']
                    icmp = stats_conn_data['icmp']
                    icmp_ingress_conn = icmp['ingress']
                    icmp_max_conn = icmp['max']
                    others = stats_conn_data['others']
                    others_ingress_conn = others['ingress']
                    others_max_conn = others['max']
                    tmp_msg = nsx_edge_fw_conn_metrics_stats_pb2.NsxEdgeFwConnSumStats(obj_id = str(fw_stats['interface']),
                              tcp_half_open_ingress_conn = tcp_half_open_ingress_conn,
                              tcp_max_conn = tcp_max_conn,
                              udp_ingress_conn = udp_ingress_conn,
                              udp_max_conn = udp_max_conn,
                              icmp_ingress_conn = icmp_ingress_conn,
                              icmp_max_conn = icmp_max_conn,
                              others_ingress_conn = others_ingress_conn,
                              others_max_conn = others_max_conn)
                    sub_msg = msg.edge_fw_conn_sum.add()
                    sub_msg.CopyFrom(tmp_msg)
                    Exportdata(data = msg, exports = ['tsdb_metrics'])
            except KeyError:
                logger.info("Keyerror, not sending edge FW conn sum to SHA")
                continue

        # export conn data from edge_fw_stats
        if fw_stats_sum:
            if precondition and precondition['value']['wavefront']['reached']:
                Exportdata(data = fw_stats_sum, exports = ['wavefront'],
                           source = wavefront_source, timestamp = timestamp)

    if edge_fw_conn is None or edge_fw_conn['value'] is None:
        return

    # Use specified source and timestamp to export metrics to wavefront
    wavefront_source = edge_fw_conn['value'][0]
    timestamp = edge_fw_conn['timestamp']

    for lr, fw_conn in edge_fw_conn['value'][1].items():

        stats_data = {}
        fw_stats_name = "edge." + lr + ".firewall"

        # firewall conn
        if fw_conn is not None:
            if 'estConn' in fw_conn:
                for key, value in fw_conn['estConn'].items():
                    if key in stats_conn_data:
                        stats_conn_data[key]['established'] = value
                    else:
                        stats_conn_data[key] = {'established': value}

            if 'tcp' in fw_conn:
                if 'tcp' in stats_conn_data:
                    stats_conn_data['tcp']['open'] = fw_conn['tcp']
                else:
                    stats_conn_data['tcp'] = {'open': fw_conn['tcp']}

            if 'flowsPerCore' in fw_conn:
                stats_data['flowsAndCores'] = fw_conn['flowsPerCore']


        if stats_conn_data:
           stats_data['connections'] = stats_conn_data

        if stats_data:
            if precondition and precondition['value']['wavefront']['reached']:
                Exportdata(data = {fw_stats_name: stats_data}, exports = ['wavefront'],
                           source = wavefront_source, timestamp = timestamp)
            if precondition and precondition['value']['tsdb']['reached']:
                lr_type, lr_name = lr.split('.')
                entity_type = options_pb2.MetricsEntityType.Value('TIER1') if lr_type == 'T1' else \
                              options_pb2.MetricsEntityType.Value('TIER0')
                msg = metrics_agent_rpc_pb2.MetricsAgentMessage(
                              timestamp = int(timestamp),
                              entity = entity_type,
                              entity_id = str(fw_conn.get('mp_router_id', 0)))
                rx_conn_per_core = 0
                tx_conn_per_core = 0
                flowsPerCore = stats_data.get('flowsAndCores', 0)
                if flowsPerCore != 0:
                    for key,flows in flowsPerCore.items():
                        if flows is not None:
                            if 'Rx' in flows:
                                rx_conn_per_core = flows['Rx']
                            if 'Tx' in flows:
                                tx_conn_per_core = flows['Tx']
                tcp = stats_conn_data.get('tcp', 0)
                if tcp != 0:
                    tcp_open_conn = tcp.get('open', 0)
                    tcp_est_conn = tcp.get('established', 0)
                else:
                    tcp_open_conn = 0
                    tcp_est_conn = 0
                udp = stats_conn_data.get('udp', 0)
                if udp != 0:
                    udp_est_conn = udp.get('established', 0)
                else:
                    udp_est_conn = 0
                icmp = stats_conn_data.get('icmp', 0)
                if icmp != 0:
                    icmp_est_conn = icmp.get('established', 0)
                else:
                    icmp_est_conn = 0
                others = stats_conn_data.get('others', 0)
                if others != 0:
                    others_est_conn = others.get('established', 0)
                else:
                    others_est_conn = 0
                tmp_msg = nsx_edge_fw_conn_metrics_stats_pb2.NsxEdgeFwConnStats(obj_id = str(fw_conn.get('interface', 0)),
                              tcp_open_conn = tcp_open_conn,
                              tcp_est_conn = tcp_est_conn,
                              udp_est_conn = udp_est_conn,
                              icmp_est_conn = icmp_est_conn,
                              others_est_conn = others_est_conn,
                              rx_conn_per_core = rx_conn_per_core,
                              tx_conn_per_core = tx_conn_per_core)
                sub_msg = msg.edge_fw_conn.add()
                sub_msg.CopyFrom(tmp_msg)
                Exportdata(data = msg, exports = ['tsdb_metrics'])

profile = data_store['profile']
edge_fw_stats = data_store['metrics']['nsx.edge.edge-fw-stats']
edge_fw_conn = data_store['metrics']['nsx.edge.edge-fw-conn']
report_edge_fw_conn(edge_fw_stats, edge_fw_conn, profile)
